"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createApi = void 0;
const utils_js_1 = require("../common/utils.js");
const errors_js_1 = require("../sdk/errors.js");
const record_js_1 = require("../sdk/record.js");
const utils_js_2 = require("./utils.js");
// POST /api/v1/records/:id @TODO
const createApi = (fetcher) => ({
    labels: {
        async find(searchParams, transaction) {
            const txId = (0, utils_js_2.pickTransactionId)(transaction);
            return fetcher(`/labels`, {
                headers: Object.assign({}, (0, utils_js_2.buildTransactionHeader)(txId)),
                method: 'POST',
                requestData: searchParams
            });
        }
    },
    properties: {
        delete: (id, transaction) => {
            const txId = (0, utils_js_2.pickTransactionId)(transaction);
            return fetcher(`/properties/${id}`, {
                headers: Object.assign({}, (0, utils_js_2.buildTransactionHeader)(txId)),
                method: 'DELETE'
            });
        },
        find: (searchParams, transaction) => {
            const txId = (0, utils_js_2.pickTransactionId)(transaction);
            return fetcher(`/properties`, {
                headers: Object.assign({}, (0, utils_js_2.buildTransactionHeader)(txId)),
                method: 'POST',
                requestData: searchParams
            });
        },
        findById: (id, transaction) => {
            const txId = (0, utils_js_2.pickTransactionId)(transaction);
            return fetcher(`/properties/${id}`, {
                headers: Object.assign({}, (0, utils_js_2.buildTransactionHeader)(txId)),
                method: 'GET'
            });
        },
        // update: () => {
        //   // @TODO
        // },
        // updateValues: (id: string, transaction?: Transaction | string) => {
        //   // @TODO
        // },
        values: (id, transaction) => {
            const txId = (0, utils_js_2.pickTransactionId)(transaction);
            return fetcher(`/properties/${id}/values`, {
                headers: Object.assign({}, (0, utils_js_2.buildTransactionHeader)(txId)),
                method: 'GET'
            });
        }
    },
    records: {
        attach: async (source, idOrIds, options, transaction) => {
            const txId = (0, utils_js_2.pickTransactionId)(transaction);
            const recordId = (0, utils_js_2.pickRecordId)(source);
            return fetcher(`/records/${recordId}/relations`, {
                headers: Object.assign({}, (0, utils_js_2.buildTransactionHeader)(txId)),
                method: 'POST',
                requestData: Object.assign(Object.assign({ targetIds: idOrIds }, ((options === null || options === void 0 ? void 0 : options.type) && { type: options === null || options === void 0 ? void 0 : options.type })), ((options === null || options === void 0 ? void 0 : options.direction) && { direction: options === null || options === void 0 ? void 0 : options.direction }))
            });
        },
        async create(data, transaction) {
            const txId = (0, utils_js_2.pickTransactionId)(transaction);
            if (data instanceof record_js_1.DBRecordDraft) {
                return fetcher(`/records`, {
                    headers: Object.assign({}, (0, utils_js_2.buildTransactionHeader)(txId)),
                    method: 'POST',
                    requestData: data.toJson()
                });
            }
            return { data: undefined, success: false };
        },
        async createMany(data, transaction) {
            const txId = (0, utils_js_2.pickTransactionId)(transaction);
            if (data instanceof record_js_1.DBRecordsBatchDraft) {
                return fetcher(`/records/import/json`, {
                    headers: Object.assign({}, (0, utils_js_2.buildTransactionHeader)(txId)),
                    method: 'POST',
                    requestData: data.toJson()
                });
            }
            return { data: [], success: false };
        },
        delete(searchParams, transaction) {
            const txId = (0, utils_js_2.pickTransactionId)(transaction);
            return fetcher(`/records/delete`, {
                headers: Object.assign({}, (0, utils_js_2.buildTransactionHeader)(txId)),
                method: 'PUT',
                requestData: searchParams
            });
        },
        deleteById(idOrIds, transaction) {
            const txId = (0, utils_js_2.pickTransactionId)(transaction);
            if ((0, utils_js_1.isArray)(idOrIds)) {
                return fetcher(`/records/delete`, {
                    headers: Object.assign({}, (0, utils_js_2.buildTransactionHeader)(txId)),
                    method: 'PUT',
                    requestData: { limit: 1000, where: { $id: { $in: idOrIds } } }
                });
            }
            else {
                return fetcher(`/records/${idOrIds}`, {
                    headers: Object.assign({}, (0, utils_js_2.buildTransactionHeader)(txId)),
                    method: 'DELETE'
                });
            }
        },
        detach: async (source, idOrIds, options, transaction) => {
            const txId = (0, utils_js_2.pickTransactionId)(transaction);
            const recordId = (0, utils_js_2.pickRecordId)(source);
            return fetcher(`/records/${recordId}/relations`, {
                headers: Object.assign({}, (0, utils_js_2.buildTransactionHeader)(txId)),
                method: 'PUT',
                requestData: Object.assign(Object.assign({ targetIds: idOrIds }, ((options === null || options === void 0 ? void 0 : options.typeOrTypes) && { typeOrTypes: options === null || options === void 0 ? void 0 : options.typeOrTypes })), ((options === null || options === void 0 ? void 0 : options.direction) && { direction: options === null || options === void 0 ? void 0 : options.direction }))
            });
        },
        export(searchParams, transaction) {
            const txId = (0, utils_js_2.pickTransactionId)(transaction);
            return fetcher(`/records/export/csv`, {
                headers: Object.assign({}, (0, utils_js_2.buildTransactionHeader)(txId)),
                method: 'POST',
                requestData: searchParams
            });
        },
        find(params, transaction) {
            const txId = (0, utils_js_2.pickTransactionId)(transaction);
            const url = (params === null || params === void 0 ? void 0 : params.id) ? `/records/${params === null || params === void 0 ? void 0 : params.id}/search` : `/records/search`;
            return fetcher(url, {
                headers: Object.assign({}, (0, utils_js_2.buildTransactionHeader)(txId)),
                method: 'POST',
                requestData: params === null || params === void 0 ? void 0 : params.searchParams
            });
        },
        findById(idOrIds, transaction) {
            const txId = (0, utils_js_2.pickTransactionId)(transaction);
            if ((0, utils_js_1.isArray)(idOrIds)) {
                return fetcher(`/records`, {
                    headers: Object.assign({}, (0, utils_js_2.buildTransactionHeader)(txId)),
                    method: 'POST',
                    requestData: { ids: idOrIds }
                });
            }
            return fetcher(`/records/${idOrIds}`, {
                headers: Object.assign({}, (0, utils_js_2.buildTransactionHeader)(txId)),
                method: 'GET'
            });
        },
        async findOne(searchParams, transaction) {
            const txId = (0, utils_js_2.pickTransactionId)(transaction);
            const response = await fetcher(`/records/search`, {
                headers: Object.assign({}, (0, utils_js_2.buildTransactionHeader)(txId)),
                method: 'POST',
                requestData: Object.assign(Object.assign({}, searchParams), { limit: 1, skip: 0 })
            });
            const [record] = response.data;
            return Object.assign(Object.assign({}, response), { data: record });
        },
        async findUniq(searchParams, transaction) {
            const txId = (0, utils_js_2.pickTransactionId)(transaction);
            const response = await fetcher(`/records/search`, {
                headers: Object.assign({}, (0, utils_js_2.buildTransactionHeader)(txId)),
                method: 'POST',
                requestData: Object.assign(Object.assign({}, searchParams), { limit: 1, skip: 0 })
            });
            if (typeof response.total !== 'undefined' && response.total > 1) {
                throw new errors_js_1.NonUniqueResultError(response.total, searchParams);
            }
            const [record] = response.data;
            return Object.assign(Object.assign({}, response), { data: record });
        },
        properties(target, transaction) {
            const txId = (0, utils_js_2.pickTransactionId)(transaction);
            const recordId = (0, utils_js_2.pickRecordId)(target);
            return fetcher(`/records/${recordId}/properties`, {
                headers: Object.assign({}, (0, utils_js_2.buildTransactionHeader)(txId)),
                method: 'GET'
            });
        },
        relations: async (target, transaction) => {
            const txId = (0, utils_js_2.pickTransactionId)(transaction);
            const recordId = (0, utils_js_2.pickRecordId)(target);
            return fetcher(`/records/${recordId}/relations`, {
                headers: Object.assign({}, (0, utils_js_2.buildTransactionHeader)(txId)),
                method: 'GET'
            });
        },
        // upsert() {
        //   // @TODO
        // }
        set(target, data, transaction) {
            const txId = (0, utils_js_2.pickTransactionId)(transaction);
            const recordId = (0, utils_js_2.pickRecordId)(target);
            return fetcher(`/records/${recordId}`, {
                headers: Object.assign({}, (0, utils_js_2.buildTransactionHeader)(txId)),
                method: 'PUT',
                requestData: data instanceof record_js_1.DBRecordDraft ? data.toJson() : data
            });
        },
        update(target, data, transaction) {
            const txId = (0, utils_js_2.pickTransactionId)(transaction);
            const recordId = (0, utils_js_2.pickRecordId)(target);
            return fetcher(`/records/${recordId}`, {
                headers: Object.assign({}, (0, utils_js_2.buildTransactionHeader)(txId)),
                method: 'PATCH',
                requestData: data instanceof record_js_1.DBRecordDraft ? data.toJson() : data
            });
        }
    },
    relations: {
        find: async (searchParams, pagination, transaction) => {
            const txId = (0, utils_js_2.pickTransactionId)(transaction);
            const queryParams = new URLSearchParams();
            if (typeof (pagination === null || pagination === void 0 ? void 0 : pagination.limit) !== 'undefined') {
                queryParams.append('limit', pagination.limit.toString());
            }
            if (typeof (pagination === null || pagination === void 0 ? void 0 : pagination.skip) !== 'undefined') {
                queryParams.append('skip', pagination === null || pagination === void 0 ? void 0 : pagination.skip.toString());
            }
            const queryString = queryParams.toString() ? '?' + queryParams.toString() : '';
            return fetcher(`/records/relations/search${queryString}`, {
                headers: Object.assign({}, (0, utils_js_2.buildTransactionHeader)(txId)),
                method: 'POST',
                requestData: searchParams
            });
        }
    },
    tx: {
        begin(config = {}) {
            return fetcher(`/tx`, {
                method: 'POST',
                requestData: config
            });
        },
        commit(id) {
            return fetcher(`/tx/${id}/commit`, {
                method: 'POST',
                requestData: {}
            });
        },
        get(id) {
            return fetcher(`/tx/${id}`, {
                method: 'GET'
            });
        },
        rollback(id) {
            return fetcher(`/tx/${id}/rollback`, {
                method: 'POST',
                requestData: {}
            });
        }
    }
});
exports.createApi = createApi;
